@@ -4,7 +4,7 @@ import mr.result_writers.base
 
 
 class InlineResultWriter(mr.result_writers.base.BaseResultWriter):
-     def get_response_tokens(self, request_id, result_pair_gen):
+     def render(self, request_id, result_pair_gen):
         return {
             'pairs': list(result_pair_gen),
         }
